@@ -2,6 +2,8 @@ use strict;
 use warnings;
 package YAML::PP::Schema::YAML1_1;
 
+our $VERSION = '0.013_001'; # TRIAL VERSION
+
 use base 'YAML::PP::Schema';
 
 use YAML::PP::Common qw/ YAML_PLAIN_SCALAR_STYLE YAML_QUOTED_SCALAR_STYLE /;
@@ -264,6 +266,16 @@ YAML::PP::Schema::YAML1_1 - YAML 1.1 Schema for YAML::PP
 
 This schema allows you to load the common YAML Types from YAML 1.1.
 
+=head1 METHODS
+
+=over
+
+=item register
+
+Called by YAML::PP::Schema
+
+=back
+
 =head1 SEE ALSO
 
 =over
